﻿using UnityEngine;

/// <summary>
/// Creates a zoomable Area within an Editor Extension.
/// 
/// Bases on:
/// http://martinecker.com/martincodes/unity-editor-window-zooming/
/// </summary>
public class EditorZoomArea
{
    /// <summary>
    /// Default Tab Height.
    /// </summary>
    private const float kEditorWindowTabHeight = 21.0f;

    /// <summary>
    /// The previous Matrix needed to reset the Matrix when ending the Zoom Area.
    /// </summary>
    private static Matrix4x4 _prevGuiMatrix;

    /// <summary>
    /// Begins a zoomable Area.
    /// </summary>
    /// <param name="zoomScale">The Zoom Scale.</param>
    /// <param name="screenCoordsArea">The Anchor Screen Coordinates and Width / Height of the Zoom Area.</param>
    /// <returns>The Zoom Area.</returns>
    public static Rect Begin(float zoomScale, Rect screenCoordsArea)
    {
        // End the group Unity begins automatically for an EditorWindow to clip out the window tab. This allows us to draw outside of the size of the EditorWindow.
        GUI.EndGroup();

        //Do the Zooming
        Rect clippedArea = screenCoordsArea.ScaleSizeBy(1.0f / zoomScale, screenCoordsArea.TopLeft());
        clippedArea.y += kEditorWindowTabHeight;
        GUI.BeginGroup(clippedArea);

        _prevGuiMatrix = GUI.matrix;
        Matrix4x4 translation = Matrix4x4.TRS(clippedArea.TopLeft(), Quaternion.identity, Vector3.one);
        Matrix4x4 scale = Matrix4x4.Scale(new Vector3(zoomScale, zoomScale, 1.0f));
        GUI.matrix = translation * scale * translation.inverse * GUI.matrix;

        //Done
        return clippedArea;
    }

    /// <summary>
    /// Ends the zoomable Area.
    /// </summary>
    public static void End()
    {
        //Reset Matrix, start the default GUI-Group again
        GUI.matrix = _prevGuiMatrix;
        GUI.EndGroup();
        GUI.BeginGroup(new Rect(0.0f, kEditorWindowTabHeight, Screen.width, Screen.height));
    }
}

