﻿using Newtonsoft.Json;
using System;
using System.IO;
using UnityEngine;

namespace Assets.Editor
{
    /// <summary>
    /// A generic Json-Parser to save and load Data.
    /// </summary>
    /// <typeparam name="T">The Model-Type used to save and load Data.</typeparam>
    internal class JsonParser<T>
    {
        /// <summary>
        /// Saves the provided Container Data persistently as a JSON File.
        /// </summary>
        /// <param name="container">The DataContainer to save.</param>
        /// <param name="fileName">The File to save the Data to.</param>
        internal void SaveData(T container, string fileName)
        {
            //Save Data to File
            try
            {
                using (StreamWriter file = File.CreateText(fileName))
                {
                    JsonSerializer serializer = new JsonSerializer();
                    serializer.Formatting = Formatting.Indented;
                    serializer.NullValueHandling = NullValueHandling.Include;
                    serializer.PreserveReferencesHandling = PreserveReferencesHandling.All;
                    serializer.Serialize(file, container);
                }
            }
            catch (Exception ex)
            {
                Debug.LogError($"An Exception occured while trying to save {fileName}: {ex.Message}{Environment.NewLine}{ex.StackTrace}");
            }
        }

        /// <summary>
        /// Loads the Data from a JSON File.
        /// </summary>
        /// <param name="fileName">The File to read the Data from.</param>
        /// <returns>The DataContainer or null, if an Error occured.</returns>
        internal T LoadData(string fileName)
        {
            //Read Json
            try
            {
                using (StreamReader file = File.OpenText(fileName))
                {
                    JsonSerializer serializer = new JsonSerializer();
                    serializer.MissingMemberHandling = MissingMemberHandling.Error;
                    serializer.NullValueHandling = NullValueHandling.Include;
                    serializer.PreserveReferencesHandling = PreserveReferencesHandling.All;
                    return (T)serializer.Deserialize(file, typeof(T));
                }

            }
            catch (Exception ex)
            {
                Debug.LogError($"An Exception occured while trying to load {fileName}: {ex.Message}{Environment.NewLine}{ex.StackTrace}");
                return default(T);
            }
        }
    }
}
